@@ -154,23 +154,14 @@ class V2Controller(wsgi.Application):
     """Base controller class for Identity API v2."""
 
     def _delete_tokens_for_trust(self, context, user_id, trust_id):
-        try:
-            token_list = self.token_api.list_tokens(context, user_id,
-                                                    trust_id=trust_id)
-            for token in token_list:
-                self.token_api.delete_token(context, token)
-        except exception.NotFound:
-            pass
+        self.token_api.delete_tokens(context, user_id,
+                                     trust_id=trust_id)
 
     def _delete_tokens_for_user(self, context, user_id, project_id=None):
         #First delete tokens that could get other tokens.
-        for token_id in self.token_api.list_tokens(context,
-                                                   user_id,
-                                                   tenant_id=project_id):
-            try:
-                self.token_api.delete_token(context, token_id)
-            except exception.NotFound:
-                pass
+        self.token_api.delete_tokens(context,
+                                     user_id,
+                                     tenant_id=project_id)
 
         #delete tokens generated from trusts
         for trust in self.trust_api.list_trusts_for_trustee(context, user_id):
