@@ -234,10 +234,15 @@ class User(controller.V2Controller):
     def update_user_project(self, context, user_id, user):
         """Update the default tenant."""
         self.assert_admin(context)
-        # ensure that we're a member of that tenant
-        default_tenant_id = user.get('tenantId')
-        self.identity_api.add_user_to_project(context,
-                                              default_tenant_id, user_id)
+
+        try:
+            # ensure that we're a member of that tenant
+            self.identity_api.add_user_to_project(
+                context, user.get('tenantId'), user_id)
+        except exception.Conflict:
+            # we're already a member of that tenant
+            pass
+
         return self.update_user(context, user_id, user)
 
 
@@ -300,7 +305,6 @@ class Role(controller.V2Controller):
 
         self.identity_api.add_role_to_user_and_project(
             context, user_id, tenant_id, role_id)
-        self._delete_tokens_for_user(context, user_id)
 
         role_ref = self.identity_api.get_role(context, role_id)
         return {'role': role_ref}
@@ -798,14 +802,6 @@ class RoleV3(controller.V3Controller):
         self.identity_api.create_grant(
             context, role_id, user_id, group_id, domain_id, project_id)
 
-        # So that existing tokens don't stop the use of this grant
-        # delete any tokens for this user or, in the case of a group,
-        # tokens from all the uses who are members of this group.
-        if user_id:
-            self._delete_tokens_for_user(context, user_id)
-        else:
-            self._delete_tokens_for_group(context, group_id)
-
     @controller.protected
     def list_grants(self, context, user_id=None, group_id=None,
                     domain_id=None, project_id=None):
